/*
  Copyright 2008 Larry Gritz and the other authors and contributors.
  All Rights Reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:
  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
  * Neither the name of the software's owners nor the names of its
    contributors may be used to endorse or promote products derived from
    this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

  (This is the Modified BSD License)
*/


/////////////////////////////////////////////////////////////////////////////
// Private definitions internal to the jpeg.imageio plugin
/////////////////////////////////////////////////////////////////////////////


#ifndef JPEG_PVT_H
#define JPEG_PVT_H


extern "C" {
#include "jpeglib.h"
}


#ifndef _TIFF_
struct TIFFDirEntry;
#endif


class JpgInput : public ImageInput {
 public:
    JpgInput () { init(); }
    virtual ~JpgInput () { close(); }
    virtual const char * format_name (void) const { return "jpeg"; }
    virtual bool open (const std::string &name, ImageSpec &spec);
    virtual bool open (const std::string &name, ImageSpec &spec,
                       const ImageSpec &config);
    virtual bool seek_subimage (int index, ImageSpec &newspec) {
        return (index == 0);   // JPEG has only one subimage
    }
    virtual bool read_native_scanline (int y, int z, void *data);
    virtual bool close ();
    std::string filename () const { return m_filename; }
    void * coeffs () const { return m_coeffs; }

 private:
    FILE *m_fd;
    std::string m_filename;
    int m_next_scanline;      // Which scanline is the next to read?
    bool m_raw;               // Read raw coefficients, not scanlines
    struct jpeg_decompress_struct m_cinfo;
    struct jpeg_error_mgr m_jerr;
    jvirt_barray_ptr *m_coeffs;

    void init () {
        m_fd = NULL;
        m_raw = false;
        m_coeffs = NULL;
    }

    // Rummage through the JPEG "APP1" marker pointed to by buf, decoding
    // IPTC (International Press Telecommunications Council) metadata
    // information and adding attributes to spec.  This assumes it's in
    // the form of an IIM (Information Interchange Model), which is actually
    // considered obsolete and is replaced by an XML scheme called XMP.
    void jpeg_decode_iptc (const unsigned char *buf);

    friend class JpgOutput;
};



namespace Jpeg_imageio_pvt {

/// Decode a raw Exif data block and save all the metadata in an
/// ImageSpec.  Return true if all is ok, false if the exif block was
/// somehow malformed.  This is a utility function to make it easy for
/// multiple format plugins to support embedding Exif metadata without
/// having to duplicate functionality within each plugin.
bool decode_exif (const void *buf, int length, ImageSpec &spec);

/// Construct an Exif data block from the ImageSpec, writing the Exif 
/// data as a big blob to the char vector.  This is a utility function 
/// to make it easy for multiple format plugins to support embedding
/// Exif data without having to duplicate functionality within each plugin.
void encode_exif (const ImageSpec &spec, std::vector<char> &exif);

};


#endif /* JPEG_PVT_H */

